@@ -33,7 +33,7 @@ NONETYPE = type(None)
 
 class Dumper:
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         """
         This dumps data structures recursively using only
         basic types, lists and dictionaries.
@@ -65,6 +65,13 @@ class Dumper:
             In most cases, it is sufficient to append new elements
             at the end, to handle more types.
 
+        These parameters can be set as named arguments in the constructor
+        or they can be set later on.
+
+        The constructor will accept any named argument, but only the documented
+        ones have any effect. This is to allow custom handlers to have their
+        own parameters as well.
+
         There is support for:
             * Basic python types (int, str, bool, float, NoneType)
             * NamedTuple
@@ -84,23 +91,39 @@ class Dumper:
         self.handlers = [
             (lambda value: type(value) in self.basictypes, lambda l, value: value),
             (lambda value: isinstance(value, tuple) and hasattr(value, '_fields') and hasattr(value, '_asdict'), _namedtupledump),
+            (lambda value: '__dataclass_fields__' in dir(value), _dataclassdump),
             (lambda value: isinstance(value, (list, tuple, set)), lambda l, value: [l.dump(i) for i in value]),
             (lambda value: isinstance(value, Enum), lambda l, value: l.dump(value.value)),
             (lambda value: isinstance(value, Dict), lambda l, value: {l.dump(k): l.dump(v) for k, v in value.items()}),
         ]  # type: List[Tuple[Callable[[Any], bool],Callable[['Dumper', Any], Any]]]
 
-    def dump(self, value: Any) -> Any:
-        for cond, func in self.handlers:
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
+
+    def index(self, value: Any) -> int:
+        """
+        Returns the index in the handlers list
+        that matches the given value.
+
+        If no condition matches, ValueError is raised.
+        """
+        for i, cond in ((j[0], j[1][0]) for j in enumerate(self.handlers)):
             try:
-                r = cond(value)
+                match = cond(value)
             except:
                 if self.raiseconditionerrors:
                     raise
-                r = False
-            if r:
-                return func(self, value)
+                match = False
+            if match:
+                return i
         raise ValueError('Unable to dump %s' % value)
 
+    def dump(self, value: Any) -> Any:
+        index = self.index(value)
+        func = self.handlers[index][1]
+        return func(self, value)
+
 
 def _namedtupledump(l, value):
     field_defaults = getattr(value, '_field_defaults', {})
@@ -109,3 +132,13 @@ def _namedtupledump(l, value):
         k: l.dump(v) for k, v in value._asdict().items()
         if not l.hidedefault or k not in field_defaults or field_defaults[k] != v
     }
+
+
+def _dataclassdump(l, value):
+    import dataclasses  # type: ignore
+    fields = set(value.__dataclass_fields__.keys())
+    field_defaults = {k: v.default for k,v in value.__dataclass_fields__.items() if not isinstance (v.default, dataclasses._MISSING_TYPE)}
+    return {
+        f: l.dump(getattr(value, f)) for f in fields
+        if not l.hidedefault or f not in field_defaults or field_defaults[f] != getattr(value, f)
+    }
