@@ -32,6 +32,25 @@ NONETYPE = type(None)
 T = TypeVar('T')
 
 
+def _issubclass(t1, t2) -> bool:
+    """
+    Wrapper around _issubclass to circumvent python 3.7 changing API
+    """
+    try:
+        return issubclass(t1, t2)
+    except TypeError:
+        return False
+
+
+# This is a workaround for an incompatibility between 3.5.2 and previous, and 3.5.3 and later
+try:
+    issubclass(Union[int,str], Union)  # type: ignore
+    HAS_UNIONSUBCLASS = True
+except:
+    HAS_UNIONSUBCLASS = False
+HAS_TUPLEARGS = hasattr(Tuple[int, int], '__args__')
+
+
 class Loader:
     """
     A loader object that recursively loads data into
@@ -77,6 +96,13 @@ class Loader:
         In most cases, it is sufficient to append new elements
         at the end, to handle more types.
 
+    These parameters can be set as named arguments in the constructor
+    or they can be set later on.
+
+    The constructor will accept any named argument, but only the documented
+    ones have any effect. This is to allow custom handlers to have their
+    own parameters as well.
+
     There is support for:
         * Basic python types (int, str, bool, float, NoneType)
         * NamedTuple
@@ -92,9 +118,9 @@ class Loader:
     similar to each other, it is easy to obtain an unexpected type.
     """
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         # Types that do not need conversion
-        self.basictypes = {int, bool, float, str}
+        self.basictypes = {int, bool, float, str, NONETYPE}
 
         # If true, it attempts to do casting of basic types
         # otherwise an exception is raised
@@ -108,21 +134,55 @@ class Loader:
         # Raise errors if the condition fails
         self.raiseconditionerrors = True
 
+        if HAS_UNIONSUBCLASS:
+            # Old python
+            union_check = lambda type_: _issubclass(type_, Union)
+        else:
+            union_check = lambda type_: getattr(type_, '__origin__', None) == Union
+
+        if HAS_TUPLEARGS:
+            tuple_check = lambda type_: getattr(type_, '__origin__', None) in {tuple, Tuple}
+        else:
+            # Old python
+            tuple_check = lambda type_: _issubclass(type_, Tuple) and _issubclass(type_, tuple) == False
+
         # The list of handlers to use to load the data.
         # It gets iterated in order, and the first condition
         # that matches is used to load the value.
         self.handlers = [
             (lambda type_: type_ == NONETYPE, _noneload),
-            (lambda type_: getattr(type_, '__origin__', None) == Union, _unionload),
+            (union_check, _unionload),
             (lambda type_: type_ in self.basictypes, _basicload),
-            (lambda type_: issubclass(type_, Enum), _enumload),
-            (lambda type_: issubclass(type_, tuple) and getattr(type_, '__origin__', None) == Tuple, _tupleload),
-            (lambda type_: issubclass(type_, list) and getattr(type_, '__origin__', None) == List, _listload),
-            (lambda type_: issubclass(type_, dict) and getattr(type_, '__origin__', None) == Dict, _dictload),
-            (lambda type_: issubclass(type_, set) and getattr(type_, '__origin__', None) == Set, _setload),
-            (lambda type_: issubclass(type_, tuple) and set(dir(type_)).issuperset({'_field_types', '_fields'}), _namedtupleload),
+            (lambda type_: _issubclass(type_, Enum), _enumload),
+            (tuple_check, _tupleload),
+            (lambda type_: getattr(type_, '__origin__', None) in {list, List}, _listload),
+            (lambda type_: getattr(type_, '__origin__', None) in {dict, Dict}, _dictload),
+            (lambda type_: getattr(type_, '__origin__', None) in {set, Set}, _setload),
+            (lambda type_: _issubclass(type_, tuple) and set(dir(type_)).issuperset({'_field_types', '_fields'}), _namedtupleload),
+            (lambda type_: '__dataclass_fields__' in dir(type_), _namedtupleload),
         ]  # type: List[Tuple[Callable[[Type[T]], bool], Callable[['Loader', Any, Type[T]], T]]]
 
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
+    def index(self, type_: Type[T]) -> int:
+        """
+        Returns the index in the handlers list
+        that matches the given type.
+
+        If no condition matches, ValueError is raised.
+        """
+        for i, cond in ((q[0], q[1][0]) for q in enumerate(self.handlers)):
+            try:
+                match = cond(type_)
+            except:
+                if self.raiseconditionerrors:
+                    raise
+                match = False
+            if match:
+                return i
+        raise ValueError('No matching condition found')
+
     def load(self, value: Any, type_: Type[T]) -> T:
         """
         Loads value into the typed data structure.
@@ -130,17 +190,14 @@ class Loader:
         TypeError is raised if there is no known way to treat type_,
         otherwise all errors raise a ValueError.
         """
-        for cond, func in self.handlers:
-            try:
-                match = cond(type_)
-            except:
-                if self.raiseconditionerrors:
-                    raise
-                match = False
-            if match:
-                return func(self, value, type_)
+        try:
+            index = self.index(type_)
+        except ValueError:
+            raise TypeError('Cannot deal with value %s of type %s' % (value, type_))
+
+        func = self.handlers[index][1]
+        return func(self, value, type_)
 
-        raise TypeError('Cannot deal with value %s of type %s' % (value, type_))
 
 def _basicload(l: Loader, value: Any, type_: type) -> Any:
     """
@@ -159,6 +216,7 @@ def _basicload(l: Loader, value: Any, type_: type) -> Any:
             raise ValueError('%s is not of type %s' % (value, type_))
     return value
 
+
 def _listload(l: Loader, value, type_) -> List:
     """
     This loads into something like List[int]
@@ -166,6 +224,7 @@ def _listload(l: Loader, value, type_) -> List:
     t = type_.__args__[0]
     return [l.load(v, t) for v in value]
 
+
 def _dictload(l: Loader, value, type_) -> Dict:
     """
     This loads into something like Dict[str,str]
@@ -175,6 +234,7 @@ def _dictload(l: Loader, value, type_) -> Dict:
     key_type, value_type = type_.__args__
     return {l.load(k, key_type): l.load(v, value_type) for k, v in value.items()}
 
+
 def _setload(l: Loader, value, type_) -> Set:
     """
     This loads into something like Set[int]
@@ -182,24 +242,38 @@ def _setload(l: Loader, value, type_) -> Set:
     t = type_.__args__[0]
     return {l.load(i, t) for i in value}
 
+
 def _tupleload(l: Loader, value, type_) -> Tuple:
     """
     This loads into something like Tuple[int,str]
     """
-    if l.failonextra and len(value) > len(type_.__args__):
+    if HAS_TUPLEARGS:
+        args = type_.__args__
+    else:
+        args = type_.__tuple_params__
+    if l.failonextra and len(value) > len(args):
         raise ValueError('Value %s is too long for type %s' % (value, type_))
-    elif len(value) < len(type_.__args__):
+    elif len(value) < len(args):
         raise ValueError('Value %s is too short for type %s' % (value, type_))
 
-    return tuple(l.load(v, t) for v, t in zip(value, type_.__args__))
+    return tuple(l.load(v, t) for v, t in zip(value, args))
+
 
 def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
     """
     This loads a Dict[str, Any] into a NamedTuple.
     """
-    fields = set(type_._fields)
-    optaional_fields = set(getattr(type_, '_field_defaults', {}).keys())
-    necessary_fields = fields.difference(optaional_fields)
+    if not hasattr(type_, '__dataclass_fields__'):
+        fields = set(type_._fields)
+        optional_fields = set(getattr(type_, '_field_defaults', {}).keys())
+        type_hints = type_._field_types
+    else:
+        #dataclass
+        import dataclasses  # type: ignore
+        fields = set(type_.__dataclass_fields__.keys())
+        optional_fields ={k for k,v in type_.__dataclass_fields__.items() if not isinstance (getattr(v, 'default', dataclasses._MISSING_TYPE()), dataclasses._MISSING_TYPE)}
+        type_hints = {k: v.type for k,v in type_.__dataclass_fields__.items()}
+    necessary_fields = fields.difference(optional_fields)
     vfields = set(value.keys())
 
     if necessary_fields.intersection(vfields) != necessary_fields:
@@ -211,10 +285,12 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
             )
         )
 
-    if l.failonextra and len(vfields.difference(fields)):
-        raise ValueError('Dictionary %s has unrecognized fields and cannot be loaded into %s' % (value, type_))
-
-    type_hints = type_._field_types
+    fieldsdiff = vfields.difference(fields)
+    if l.failonextra and len(fieldsdiff):
+        extra = ', '.join(fieldsdiff)
+        raise ValueError(
+            'Dictionary %s has unrecognized fields: %s and cannot be loaded into %s' % (value, extra, type_)
+        )
 
     params = {}
     for k, v in value.items():
@@ -223,6 +299,7 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
         params[k] = l.load(v, type_hints[k])
     return type_(**params)
 
+
 def _unionload(l: Loader, value, type_) -> Any:
     """
     Loads a value into a union.
@@ -234,14 +311,21 @@ def _unionload(l: Loader, value, type_) -> Any:
     If no suitable type is found, an exception is raised.
     """
 
+    if hasattr(type_, '__args__'):
+        args = type_.__args__
+    elif hasattr(type_, '__union_params__'):
+        args = type_.__union_params__
+    else:
+        raise AttributeError('The typing API for this Python version is unknown')
+
     # Do not convert basic types, if possible
-    if type(value) in set(type_.__args__).intersection(l.basictypes):
+    if type(value) in set(args).intersection(l.basictypes):
         return value
 
     exceptions = []
 
     # Try all types
-    for t in type_.__args__:
+    for t in args:
         try:
             return l.load(value, t)
         except Exception as e:
@@ -254,6 +338,7 @@ def _unionload(l: Loader, value, type_) -> Any:
         )
     )
 
+
 def _enumload(l: Loader, value, type_) -> Enum:
     """
     This loads something into an Enum.
@@ -280,6 +365,7 @@ def _enumload(l: Loader, value, type_) -> Enum:
             pass
     raise ValueError('Value %s could not be loaded into %s' % (value, type_))
 
+
 def _noneload(l: Loader, value, type_) -> None:
     """
     Loads a value that can only be None,
